;; prostscript.ps - routines for creating a posscript file
;;
;; version 0.91 - L.M. July 2006
;;
;; Postscript files can be viewed using: filename.ps on Mac OS X
;; or using the Ghostscript program on Unix's or Win32 to convert
;; to PDF or any graphics file format. Best quality is achieved
;; on Mac OS X when using the Preview.app viewer for loading the 
;; postscript files and converting to PDF or bitmapped formats like 
;; JPEG, PNG, GIF by re-saving.
;;
;; If not using Mac OS X look for Ghostscript here:
;;    http://www.ghostscript.com/
;;    http://www.cs.wisc.edu/~ghost/
;;
;; NOTE! on some Mac OS X installations it is necessary to quit out of
;; the Preview.app completely before viewing a .ps file for the first
;; time. Subsequent views of .ps documents are fine.
;;
;; On Linux/UNIX systems the following command can be used to convert
;; a .ps file to a .pdf file:
;;
;;   gs -sDEVICE=pdfwrite -dBATCH -sOutputFile=aFile.pdf -r300 aFile.ps
;;
;; Most functions work like in turtle graphics relative to the
;; current position of an imaginary drawing pen with an
;; orientation of 0 to 360 degree starting streight up: 0 and
;; moving clockwise right: 90, down: 180, left: 270, up: 360.
;; Other functions works on the absolute X,Y coordinates.
;;
;; The coordinate system starts on the bottom left with 0,0 and
;; extends on a 8 1/2 x 11 inch letter page to x: 612, y: 792,
;; 12 points for each inch. The functions (transpose x y) and (scale x y)
;; can be used to move the origin point x=0,y=0 or scale from points to
;; other measurements.
;;
;;
;; Function API:
;;
;; TURTLE COORDINATE POSITIONING TURNING
;;
;; All commands in this group change the turtle position or 
;; orientation.
;;
;; (ps:goto x y) - move to position x,y
;;    x - new x coordinate
;;    y - new y coordinate
;;    On a US letter page 612 by 792 point positions
;;    are defined, 72 points per inch
;;
;; (ps:move s) - move turtle forward distancde s
;;    Like ps:draw bit does not draw
;;
;; (ps:turn dg) - turn the turtle pen by dg degrees
;;    dg - degrees to turn -380 to 0 to 360
;;    The degrees are specified in angles from 0 to 360
;;    For turning clock wise specifiy positive values.
;;    Negative degrees turn the turtle pen counter clock wise.
;;
;; (ps:angle dg) - set the turtle angle to dg degree
;;    dg - angle degrees from 0 to 360
;;    Sets the turtle pen orientation to the angle specified
;;    in dg in degrees between 0 and 360:
;;    Upwards 0, right 45, downwards 180, left 270.
;;
;; LINE DRAWING
;;
;; All commands in this group change the turtle position or 
;; orientation, or both.
;;
;; (ps:draw s) - draw going forward distance s
;;    Moves the turtle forward by the amoint of points specified
;;    in s and draws with the current line-color.
;;
;; (ps:drawto x y) - draw a line to point x,y
;;    Moves the turtle to point x, y like (ps:goto x y), but
;;    also draws a line from the old to the new position.
;;    The turtle position changed to the new point x,y and the 
;;    orientation is changed to the orientaion of the line drawn.
;;
;; (ps:line '((...) (...) ...)) - draw a multipart line
;;    (...) - is a turtle movement (dg s) or
;;            a Bezier curve (x1 y1 x2 y2 x3 y3) starting
;;	          from the last turtle coordinates x0 y0 and
;;            finishing at x3 y3. All Bezier cooerdinates
;;            relative to the previous turtle position and
;;            orientation.
;;    The turtel position and orientation are changed after
;;    drawing the line.
;;
;; (ps:bezier x1 y1 x2 y2 x3 y3) - draw a Bezier curve
;;    x1, y1 - Bezier coordinates of p1 relative to p0 = 0,0
;;    x2, y2 - Bezier coordinates of p1 relative to p0 = 0,0
;;    x3, y3 - Bezier coordinates of p1 relative to p0 = 0,0
;;    The Bezier curve starts at point p0 which is the current
;;    turtle position and stops at point p3 which is at offset
;;    x3 and y3 relative to starting point. The turtle orientation
;;    after the drwaing the Bezier curve is perpendicular
;;    to the Bezier curve baseline p0 to p3.
;;
;; CLOSED SHAPES, FILLING AND CLIPPING
;;
;; All functions in this group leave the turtle position and
;; orientation unchanged.
;;
;; (ps:polygon rad n [fill]) - draw a polygon with
;;    rad - radius 
;;    n - sides 
;;    fill - true, false (default) for optional color fill
;;    The polygon is drawn around the current turtle position.
;;    The turtle position or orientation is not changed.
;;
;; (ps:circle rad [fill]) - draw a circle with
;;    rad - radius, 
;;    fill - true, false (default) for optional color fill
;;    The circle is drawn around the current turtle position.
;;    The turtle position or orientation is not changed.
;;
;; (ps:ellipse xrad yrad start end [fill]) - draw an ellipse with
;;    xrad - x axis radius
;;    yrad - y axis radius
;;    start - start angle
;;    end - end angle
;;    fill - true, false (default) for optional color fill
;;    The ellipse is drawn around the current turtle position
;;    with the Y axis oriented like the turtle.
;;    For xrad, yrad set to 0, 360 an ellipse is drawn
;;    For a partiual radius the opening is closed by a line
;;    resulting in segment shape, i.e. -90, 90  would result
;;    in a half circle from the left to the right of the turtle.
;;    The turtle position or orientation is not changed.
;;
;; (ps:pie rad width [fill]) - draw a pie slice
;;    rad - radius
;;    width - width angle
;;    fill - true, false (default) for optional color fill
;;    The left edge of the pie is in tutrlte orientation.
;;    The width angle spreads clockwise.
;;    The pie is drawn around the current turtle position.
;;    The turtle position or orientation is not changed.
;;
;; (ps:petal x y [fill]) - draws a petal using a Bezier curve
;;    x - x1 and x2 of underlying Bezier curve p0 -> p1 p2 p3
;;    y - y1 and y2 of underlying Bezier curve p0 -> p1 p2 p3
;;    fill - true, false (default) for optional color fill
;;    The x and y parameters are relative to to the current position.
;;    The petal is drawn with the tip at the current turtle
;;    position and oriented in the direction of the turtle.
;;    The turtle position or orientation is not changed.
;;
;; (ps:shape '((...) (...) ...) [fill]) - draw a shape
;;    (...) - is a turtle movement (dg s) or
;;            a Bezier curve (x1 y1 x2 y2 x3 y3) starting
;;	          from the last turtle coordinates x0 y0 and
;;            finishing at x3 y3. All Bezier cooerdinates
;;            relative to the previous turtle position and
;;            orientation
;;    fill - true, false (default) for optional color fill
;;    The turtle position or orientation is not changed.
;;
;; (ps:clip '((...) (...))) - define a clipping path
;;    (...) - is a turtle movement (dg s) or
;;            a Bezier curve (x1 y1 x2 y2 x3 y3) starting
;;	          from the last turtle coordinates x0 y0 and
;;            finishing at x3 y3. All Bezier cooerdinates
;;            relative to the previous turtle position and
;;            orientation.
;;    Before redefining the clipping area (ps:gsave) should
;;    be used to save the old graphics state parameters, after
;;    clipping and drawing in the clipped area the graphics
;;    state should be restored using (ps:grestore).
;;    The turtle position or orientation is not changed.
;;
;; TEXT OUTPUT AND CLIPPING
;; 
;; All functions in this goup move the turtle by the textlength
;; drawn and change the orientation when the text is arcing.
;;
;; (ps:text str) - draw a text
;;    str - text to draw
;;    (,) parenthesis in text should be escaped with
;;    double \\ chracters a sin in \\( or \\), when limiting the string
;;    with double quotes ". When limiting the string with {,} braces
;;    a single \ character is enough as in \( and \).
;;    Before drawing a font can be specified, the default font after loading 
;;    the postscript.lsp modules is Helvetica 12 points and using
;;    the current line-color for drawing.
;;    The turtle position is changes to the baseline after the last character.
;;    The turtle orientation is the same.
;;
;; (ps:textoutline str [fill]) - draw a text outline
;;    str  - text to draw 
;;    fill - true, false (default) for optional color fill
;;    Before drawing a font can be specified
;;    the default font after loading postscript.ps is
;;    Helvetica 12points, the text is drawn using the current
;;    line color.
;;
;; (ps:textarc rad str) - draw text around a circle
;;    rad - radius of imaginary circle path for text
;;    str - text to draw 
;;    The text is drawn out side of an imaginary circle starting at 
;;    turtle position and orientation and drawing at the current tangent.
;;    To bend text around a circle, draw a sequence of single characters
;;    with the same radius. For a positive radius text goes outside
;;    the curcle and clockwise. For a negative radius text goes inside the
;;    circle and counter lock wise. The turtle position and orientation
;;    move along the radius.
;;
;;
;; (ps:textarcoutline rad str) - draw text around a circle
;;    rad - radius of imaginary circle path for text
;;    str - text to draw
;;    fill - true, false (default) for optional color fill
;;    Same as (ps:textarc ...) but the text is drawn as ane outline
;;    and can be filed with fill-color. The turtle position and orientation
;;    move along the radius.
;;
;; (ps:textclip str)
;;    str - text used as a clipping shape
;;    A text outline is used as a clipping path.
;;    Before redefining the clipping area (ps:gsave) should
;;    be used to save the old graphics state parameters, after
;;    clipping and drawing in the clipped area the graphics
;;    state should be restored using (ps:grestore).
;;    The turtle moves with the text shape clipped.
;;
;; GLOBAL SETTINGS
;;
;; (ps:font name size) - set a font
;;    name = font name in a string
;;    size = size of the font in points
;; The current font is set for all subsequent text operations
;; Depending on the version of the Postsrcipt viewer or device
;; installed different fonts are available.
;;
;; (ps:translate x y) - move the coordinate origin
;;    x - new x position of coordinate origin
;;    y - new y position of coordinate origin
;;    By default the origin 0,0 is in the bottom left corner
;;    of the page. x and y values extend to the right and top. 
;; (ps:translate) - move coordinate origin to current position
;;
;; (ps:scale x y) - scale the coordinate space
;;    x - new x scale factor
;;    y - new y scale factor
;;    Sacling factors are 1.0 by default and compress for
;;    factors less 1.0 or expand for factors bigger than 1.0
;;    With a scaling factor for x = 2.0 each point position
;;    specified would cover the double of horizontal distance
;;    on the page. Scaling factors can be saved on the graphics
;;    state stack using the function (ps:gsave).
;;
;; (ps:rotate deg) - rotate the coordinate space
;;   deg - degree of rotation, -360 to 0 to 360
;;   The coorinate space is rotated to the right for
;;   positive angles and to the left for negative anggle.
;;   The current rotation angle is 0 by default.
;;   The rotation angle is part of the graphics state saved by
;;   the (ps:gsave function).
;;
;; (ps:gsave) - save graphics state
;;   Saves the current graphics state. The function pushes the 
;;   current graphics state on a special stack, from where it
;;   can be resored using (ps:grestore).
;;
;; (ps:grestore) - restore graphics state
;;    Restores the graphics state from the stack.
;;
;; (ps:line-width points) - set line width
;;    Sets the line width in points for line drawing and the
;;    outlines drawn by shapes and text outlines.
;;
;; (ps:line-cap mode) - set mode for line ends
;;    mode - 0, 1, 2
;;    Sets the line termination shape.
;;    0 square line at the end
;;    1 semicircular line
;;    2 square line end projecting beyond the end of the line
;;      by half line width
;;
;; (ps:line-join mode) - set join mode for lines
;;    mode - 0, 1, 2
;;    0 outer edges of lines mitered together
;;    1 outer edges of lines rounded together
;;    2 for lin-cap with 0 the resulting noth
;;      is filled to produce a chamfered corner
;;
;; (ps:line-color r g b) - set color for line drawing
;; (ps:line-color str-hex)
;;    r - value for red 0.0 to 1.0
;;    b - value for green 0.0 to 1.0
;;    g - value for blue 0.0 to 1.0
;;    str-hex - hex string contant "000000" to "FFFFFF"
;;    Coosr are specified as usual in HTML coding.
;;    Each to two hex digits define a color: rrggbb.  
;;
;; (ps:fill-color r g b) - set color for shape filling
;; (ps:fill-color str-hex)
;;    r - value for red 0.0 to 1.0
;;    b - value for green 0.0 to 1.0
;;    g - value for blue 0.0 to 1.0
;;    str-hex - hex string contant "000000" to "FFFFFF"
;;    Coosr are specified as usual in HTML coding.
;;    Each to two hex digits define a color: rrggbb.
;;
;; (ps:render) - show on monitor (Mac OS X only)
;;    Uses the Mac OS X Preview.app to convert
;;    and view postscript files ending in .ps.
;;    On Unix and Win32 systems use (ps:save <filename>)
;;    to save the postscript file and convert and view
;;    it using ghostscript from http://www.ghostscript.com/
;;    and http://www.cs.wisc.edu/~ghost/
;;
;; (ps:save str-filename) - save to filename
;;    str-filename - filename in a string 
;;    The filename should end in .ps to be recognized
;;    as a Postscript file on Mac OS X, where it can
;;    be viewed with the standrad Preview.app, by
;;    double clicking the filename. On Linus/UNIX
;;    gostsript can be used to convert the file to
;;    to any image format or to view the file.
;;    Quality of display depends on underlying OS
;;    and hardware.
;;

(context 'ps)

(set 'prolog [text]%!PS-Adobe-2.0
%%Creator: newLISP

%% ---------- SETUP ----------

/orient 0 def
/xpos 0 def
/ypos 0 def
/pi 3.141592654 def

/fillcolor {0.8 0.8 0.8} def
/Helvetica findfont 12 scalefont setfont

/turtlestack [0 0 0] def

/pushturtle
	{	
	turtlestack length /len exch def 
	turtlestack aload pop 
	xpos ypos orient len 3 add array astore
	/turtlestack exch def
	} def

/popturtle
	{
	turtlestack length /len exch def
	len 3 gt {
		turtlestack aload pop
		/orient exch def
		/ypos exch def
		/xpos exch def
		len 3 sub array astore
		/turtlestack exch def
		} if
	} def


%% ---------- NAVIGATION ----------

% x y -
/goto
  {
  /ypos exch def
  /xpos exch def
  xpos ypos moveto
  } def

% points -
/move
  {
  /len exch def
  /xpos xpos orient sin len mul add def
  /ypos ypos orient cos len mul add def
  xpos ypos moveto
  } def
  
% degree -
/turn
  {
  /orient exch orient add def
  } def
  
% degree -
/angle
  {
  /orient exch def
  } def
  
%% ---------- LINE DRAWING ----------

% turtle position is changed
  
% points -
/draw
  {
  /len exch def
  newpath
  xpos ypos moveto
  /xpos xpos orient sin len mul add def
  /ypos ypos orient cos len mul add def
  xpos ypos lineto stroke
  } def  

% points -
/drawtopoint
  {
  /len exch def
  /xpos xpos orient sin len mul add def
  /ypos ypos orient cos len mul add def
  xpos ypos lineto
  } def  
  
% x y
/drawto
  {
  /newy exch def
  /newx exch def
  newpath
  xpos ypos moveto
  newx newy lineto
  stroke
  newy ypos sub newx xpos sub atan neg 90 add /orient exch def
  /xpos newx def
  /ypos newy def
  } def
  
% x1 y1 x2 y2 x3 y3
/bezier
  {
  newpath
  curve
  stroke
  } def

/curve
  {
  /y3 exch def
  /x3 exch def
  /y2 exch def
  /x2 exch def
  /y1 exch def
  /x1 exch def
  matrix currentmatrix
  x1 y1 x2 y2 x3 y3
  xpos ypos translate
  orient neg rotate
  0 0 moveto
  rcurveto
  setmatrix
  y3 x3 atan neg /angleinc exch def
  /len x3 angleinc cos div def
  /orient orient angleinc add def
  /xpos xpos orient 90 add sin len mul add def
  /ypos ypos orient 90 add cos len mul add def
  } def

% save turtle position and orientation

/turtlesave
  {
  /xpossave xpos def
  /ypossave ypos def
  /orientsave orient def
  } def
  
% restore turtle position and orientation

/turtlerestore
  {
  /xpos xpossave def
  /ypos ypossave def
  /orient orientsave def
  xpos ypos moveto
  } def

% x1 y1 x2 y2 -
/fromto
  {
  /ypos exch def
  /xpos exch def
  newpath
  moveto
  xpos ypos lineto
  stroke
  } def

%% ---------- SHAPES ----------

% shapes are close and do not change the turtle position

% radius sides fillflag -
/polygon
  {
  /fillflag exch def
  360 exch div /orientinc exch def
  /radius exch def
  gsave
  xpos ypos translate
  orient neg rotate
  0 sin radius mul
  0 cos radius mul moveto
  0 orientinc 360
   {
   dup
   sin radius mul exch
   cos radius mul
   lineto
   } for
  closepath
  fillflag {fillshape} if
  stroke
  grestore
  } def
  
% radius fillflag -
/circle
  {
  /fillflag exch def
  /radius exch def
  newpath
  xpos ypos radius 0 360 arc
  fillflag {fillshape} if
  stroke
  } def
  
  
% radius width fillflag
/pie
  {
  /fillflag exch def
  /width exch def
  90 orient sub width sub /start exch def
  start width add /end exch def
  /radius exch def
  newpath
  xpos ypos moveto
  xpos ypos radius start end arc
  fillflag {fillshape} if
  closepath
  stroke
  } def
  
% width height fill
/petal
  {
  /fillflag exch def
  /y exch def
  /x exch def
  gsave
  xpos ypos translate
  orient neg rotate
  newpath
  0 0 moveto
  x neg y x y 0 0 
  rcurveto
  fillflag {fillshape} if
  closepath
  stroke
  grestore
  } def
  
% xradius yradius start end flag -
/ellipse
  {
  /fillflag exch def
  % swap start/end and x/y
  neg /startangle exch def
  neg /endangle exch def
  /xrad exch def
  /yrad exch def
 
  gsave
  xpos ypos translate
  orient 90 sub neg rotate
  newpath
  xrad yrad scale
  0 0 1 startangle endangle arc
  fillflag {fillshape} if
  1 xrad div 1 yrad div scale
  closepath
  stroke
  grestore
  } def

/fillshape
  {
  gsave
  fillcolor setrgbcolor
  fill 
  grestore
  } def
   
%% ---------- text ----------

/text
  {
  /str exch def
  gsave
  xpos ypos translate
  newpath
  0 0 moveto
  orient 89.9999 sub neg rotate
  str show
  grestore
  str stringwidth pop move
  } def
  
/textoutline
  {
  /fillflag exch def
  /str exch def
  gsave
  xpos ypos translate
  newpath
  0 0 moveto
  orient 89.9999 sub neg rotate
  str true charpath
  fillflag {fillshape} if
  stroke
  grestore
  str stringwidth pop move
  } def
  
/textclip
  {
  /str exch def
  matrix currentmatrix
  xpos ypos translate
  newpath
  0 0 moveto
  orient 89.9999 sub neg rotate
  str true charpath
  clip
  setmatrix
  } def

/textarc
  {
  /str exch def
  2 mul pi mul /circum exch def
  
  str stringwidth pop /len exch def
  circum len div 360 exch div turn
  str text
  } def
  
/textarcoutline
  {
  /fillflag exch def
  /str exch def
  2 mul pi mul /circum exch def
  
  str stringwidth pop /len exch def
  circum len div 360 exch div turn
  str fillflag textoutline
  } def
  
% --------------------------
[/text])

;; ---------- setup ---------- 

(set 'buffer "")
(set 'line-feed (if (> (& 0xF (sys-info -1)) 5) "\r\n" "\n"))

;; ---------- USER FUNCTIONS ----------

;; ---------- output pure postscript ---------- 

(define (ps:ps str)
	(write-line str buffer))
	
;; navigation - changes position or orient of the turtle
	
(define (goto x y)
	(ps (format "%d %d goto" x y)))
	
(define (turn deg)
	(ps (format "%f turn" deg)))
	
(define (move dist)
	(ps (format "%f move" dist)))
	
(define (angle deg)
	(ps (format "%f angle" deg)))
	
;; line graphics changes position and/or orient of the turtle
	
(define (draw dist)
	(ps (format "%f draw" dist)))
	
(define (drawto x y)
	(ps (format "%f %f drawto" x y)))
	
(define (bezier x1 y1 x2 y2 x3 y3)
	(ps (format "%d %d %d %d %d %d bezier" x1 y1 x2 y2 x3 y3)))
	
(define (line lst)
	(let (rec nil)
		(ps "% new shape")
		(ps "newpath")
		(ps "xpos ypos moveto")
		(while (set 'rec (pop lst))
			(if (= (length rec) 6)
				(ps (format "%f %f %f %f %f %f curve"
						(rec 0) (rec 1) (rec 2)
						(rec 3) (rec 4) (rec 5)))
				(begin
					(ps (format "%f turn" (rec 0)))
					(ps (format "%f drawtopoint" (rec 1))))))
		(ps "stroke")))
	
;; shapes do not change the position or orient of the turtle
;; which stays in the curcle center of the shape

(define (polygon radius sides flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "%d %d %s polygon" radius sides flag)))
	
(define (circle radius flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "%d %s circle" radius flag)))
	
(define (ellipse xradius yradius start end flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "%d %d %d %d %s ellipse" xradius yradius start end flag)))
	
(define (pie radius width flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "%d %d %s pie" radius width flag)))
	
(define (petal width height flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "%d %d %s petal" width height flag)))
	
(define (shape lst flag)
	(let (rec nil)
		(ps "% new shape")
		(ps "turtlesave")
		(ps "newpath")
		(ps "xpos ypos moveto")
		(while (set 'rec (pop lst))
			(if (= (length rec) 6)
				(ps (format "%f %f %f %f %f %f curve"
						(rec 0) (rec 1) (rec 2)
						(rec 3) (rec 4) (rec 5)))
				(begin
					(ps (format "%f turn" (rec 0)))
					(ps (format "%f drawtopoint" (rec 1))))))
		(ps "closepath")
		(if flag (ps "fillshape"))
		(ps "stroke")
		(ps "turtlerestore")))

(define (clip lst)
	(let (rec nil)
		(ps "% new clipping shape")
		(ps "turtlesave")
		(ps "newpath")
		(ps "xpos ypos moveto")
		(while (set 'rec (pop lst))
			(if (= (length rec) 6)
				(ps (format "%f %f %f %f %f %f curve"
						(rec 0) (rec 1) (rec 2)
						(rec 3) (rec 4) (rec 5)))
				(begin
					(ps (format "%f turn" (rec 0)))
					(ps (format "%f drawtopoint" (rec 1))))))
		(ps "closepath")
		(ps "clip"))
		(ps "turtlerestore"))
		
;; text output 

(define (text str)
	(ps (format "(%s) text" str)))
	
(define (textoutline str flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "(%s) %s textoutline" str flag)))
	
(define (textclip str)
	(ps (format "(%s) textclip" str)))
		
(define (textarc radius str)
	(ps (format "%f (%s) textarc" radius str)))
	
(define (textarcoutline radius str flag)
	(set 'flag (if flag "true" "false"))
	(ps (format "%f (%s) %s textarcoutline" radius str flag)))
	
;; rendering and saving

(define (render)
	(write-file "noname.ps" (append prolog buffer "showpage" line-feed))
	(exec "open noname.ps"))
	
(define (clear)
	(set 'buffer ""))
	
(define (ps:save file-name)
	(write-file file-name (append prolog buffer "showpage" line-feed)))
	
;; global parameters 

(define (translate x y)
	(if (and x y)
		(ps (format "%d %d translate" x y))
		(ps "xpos ypos translate 0 0 moveto")))
	
(define (scale x y)
	(ps (format "%f %f scale" x y)))
	
(define (ps:rotate deg)
	(ps (format "%d rotate" deg)))

(define (gsave)
	(ps "pushturtle gsave"))
	
(define (grestore)
	(ps "grestore popturtle"))
	
(define (line-width points)
	(ps (format "%f setlinewidth" points)))
	
(define (line-cap mode)
	(ps (format "%d setlinecap")))
	
(define (line-join mode)
	(ps (format "%d setlinejoin")))
	
(define (line-color red green blue)
    (if (string? red)
    	(let (color red)
    	  (set 'red (div (int (append "0x" (0 2 color)) 0 16) 255))
    	  (set 'green (div (int (append "0x" (2 2 color)) 0) 255))
    	  (set 'blue (div (int (append "0x" (4 2 color)) 0) 255))))
	(ps (format "%f %f %f setrgbcolor" red green blue)))
	
(define (fill-color red green blue)
    (if (string? red)
    	(let (color red)
    	  (set 'red (div (int (append "0x" (0 2 color)) 0 16) 255))
    	  (set 'green (div (int (append "0x" (2 2 color)) 0) 255))
    	  (set 'blue (div (int (append "0x" (4 2 color)) 0) 255))))
	(ps (format "/fillcolor {%f %f %f} def" red green blue)))
	
(define (font fname size)
	(ps (format "/%s findfont %d scalefont setfont" fname size)))
	
(context MAIN)




	



